randomForest.balanced<- function(dat, B, Nsel) {
  library(randomForest)
  conflict_prefer("margin", "randomForest")
  set.seed(7636)
  
  ## Code adapted from Arevalillo et.al. (2017) "https://doi.org/10.1016/j.jbi.2017.08.005"
  
  #######################################################################
  #ASSESSING VARIABLE IMPORTANCE BY BOOTSTRAPING RF TABLE OF IMPORTANCES#
  #######################################################################
  
  #dat is the data frame containing your input data
  
  #.....................................................................................................................
  #Data preparation. Loop for deletion of NA (if desire for running RF. Another possibility is to set na.action=na.omit)
  #......................................................................................................................
  
  numloop<-ncol(dat)
  
  for (i in 1:numloop)
  {
    dat<-dat[!is.na(dat[,i]),]
    # cat(i,fill=TRUE)
  }
  
  #......................................................................................................
  #End of Data preparation
  #......................................................................................................
  
  RankingMatrix<-matrix(0,B,ncol(dat)-1)
  colnames(RankingMatrix)<-colnames(dat[,c(1:ncol(dat)-1)])
  Err <- rep(0, B)
  out = list()
  for (i in 1:B){
    datatemp<-dat %>% group_by(Y) %>% slice_sample(n = Nsel)
    rf<-randomForest(as.factor(Y)~.,data=datatemp,ntree=1000,importance=TRUE,na.action=na.omit)
    RankingMatrix[i,]<-rf$importance[,"MeanDecreaseAccuracy"]
    Err[i]<- round(rf$err.rate[nrow(rf$err.rate),1] * 100,2)
  }
  
  Importance.Scores<-100*(apply(RankingMatrix,2,mean)-
                            min(apply(RankingMatrix,2,mean)))/diff(range(apply(RankingMatrix,2,mean)))
  Importance.Scores = sort(Importance.Scores, decreasing = TRUE)
  out[[1]] = Importance.Scores
  out[[2]] = Err
  
  return(out)
}